import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'variables.dart';
import 'function.dart';
import 'dart:convert';

void getDevices() async {
  try {
    var temp = await Storage.controller!.get(':Devices');
    var getChildNodes = await temp.getChildNodesCsv();
    var devicesUUIDs = getChildNodes.toString().split(',');

    for (var uuid in devicesUUIDs) {
      var deviceNodesTemp = await Storage.controller!.get(':Devices:$uuid');
      var getChildren = await deviceNodesTemp.getChildren();
      //print(getChildren);
      //print(storage.userUUID);
      var deviceNodes = getChildren.entries.iterator;

      //await deviceNodesTemp.getChildren().asStream().forEach((element) { deviceNodes = element.entries.iterator;});
      //entries.iterator;

      try {
        deviceNodes.moveNext();
        var ownerNode = await deviceNodes.current.value.getValue('owner');
        if (ownerNode!.getValue('en') == Storage.userUUID) {
          var deviceScoreNode = await Storage.controller!.get(
              ':Devices:$uuid:' +
                  Storage.geigerIndicatorUUID +
                  ':data:GeigerScoreDevice');

          var geigerDevScore = deviceScoreNode.getValue('GEIGER_score');

          var geigerDevNumMetrics = deviceScoreNode.getValue('number_metrics');

          Node scoreNum = NodeImpl('', '');
          await scoreNum
              .addValue(NodeValueImpl('score', geigerDevScore.toString()));
          await scoreNum
              .addValue(NodeValueImpl('nums', geigerDevNumMetrics.toString()));
          CoreValues.aggregate.add(scoreNum);
        }
      } catch (e) {
        continue;
      }
    }
    // ignore: empty_catches
  } catch (e) {}
}

void getEmployees() async {
  try {
    var userNode =
        await Storage.controller!.get(':Enterprise:users:' + Storage.userUUID);

    var mainUserNode = await userNode.getValue('mainUser');
    var flagmainUser = mainUserNode!.getValue('en');

    var supervisingNode = await userNode.getValue('supervising');
    var employeeNode = supervisingNode!.getValue('en');

    CoreValues.flagMainUser = flagmainUser == '1' ? true : false;

    if (employeeNode != null) {
      var employees = employeeNode.split(',');

      for (var empUUID in employees) {
        var userAggregateScoreNodeName = ':Users:' +
            empUUID +
            ':' +
            Storage.geigerIndicatorUUID +
            ':data:GeigerScoreAggregate';

        try {
          var userAggregateScoreNode =
              await Storage.controller!.get(userAggregateScoreNodeName);

          var scoreNode = await userAggregateScoreNode.getValue('GEIGER_score');
          var geigerAggScore = scoreNode!.getValue('en')!;

          var metricNode =
              await userAggregateScoreNode.getValue('number_metrics');
          var geigerAggNumMetrics = metricNode!.getValue('en')!;

          Node scoreNums = NodeImpl('', '');
          await scoreNums.addValue(NodeValueImpl('score', geigerAggScore));
          await scoreNums.addValue(NodeValueImpl('nums', geigerAggNumMetrics));
          CoreValues.aggregate.add(scoreNums);
        } catch (e) {
          continue;
        }
      }
    }
  } catch (e) {
    // ignore: avoid_print
    print(e);
    return;
  }
}

Future getEnterpriseUUID() async {
  try {
    var enterpriseNode = await Storage.controller!.get(':Enterprise');

    var getChildren = await enterpriseNode.getChildren();

    var enterpriseNodeIterator = getChildren.entries.iterator;
    var profiles = [];

    while (enterpriseNodeIterator.moveNext()) {
      try {
        var sectorNode =
            await enterpriseNodeIterator.current.value.getValue('sector');
        var sector = sectorNode!.getValue('en');

        var locationNode =
            await enterpriseNodeIterator.current.value.getValue('location');
        var location = locationNode!.getValue('en');

        var profileNode = await enterpriseNodeIterator.current.value
            .getValue('associatedProfiles');
        var profile = profileNode!.getValue('en');
        profiles.add(profile);

        CoreValues.enterpriseNode['sector'] = sector;
        CoreValues.enterpriseNode['location'] = location;
        CoreValues.enterpriseNode['associatedProfiles'] = profile;
      } catch (e) {
        //print(e);
        continue;
      }
    }

    return profiles;
  } catch (e) {
    // ignore: avoid_print
    print(e);
  }
}

Future getRisk() async {
  try {
    var profilesNode = await Storage.controller!.get(':Global:profiles');

    var getChildren = await profilesNode.getChildren();

    var profileNodeIterator = getChildren.entries.iterator;

    while (profileNodeIterator.moveNext()) {
      var nameNode = await profileNodeIterator.current.value.getValue('name');

      var profileName = nameNode!.getValue('en')!.toLowerCase();

      for (String threatUUID in CoreValues.uuidThreats) {
        var threatUUIDNode =
            await profileNodeIterator.current.value.getValue(threatUUID);

        switch (profileName) {
          case Types.digitallyBased:
            CoreValues.based[threatUUID] = threatUUIDNode!.getValue('en')!;
            break;

          case Types.digitalEnabler:
            CoreValues.enabler[threatUUID] = threatUUIDNode!.getValue('en')!;
            break;

          case Types.digitallyDependent:
            CoreValues.dependent[threatUUID] = threatUUIDNode!.getValue('en')!;
            break;
        }
      }
    }
  } catch (e) {
    // ignore: avoid_print
    print(e);

    return;
  }
}

Future getRecommendation() async {
  try {
    var recommendationNode =
        await Storage.controller!.get(':Global:Recommendations');

    var getChildren = await recommendationNode.getChildren();

    var recoNodeIterator = getChildren.entries.iterator;

    while (recoNodeIterator.moveNext()) {
      var recommendationTypeNode =
          await recoNodeIterator.current.value.getValue('RecommendationType');

      var recommendationType = recommendationTypeNode!.getValue('en')!;

      var relatedThreatsWeightsNode = await recoNodeIterator.current.value
          .getValue('relatedThreatsWeights');
      var lists =
          relatedThreatsWeightsNode!.getValue('en')!.split(';'); //List<String>

      for (var node in lists) {
        var relatedThreatsWeights = node.split(',');

        Node impactNormal = NodeImpl('', '');
        await impactNormal
            .addValue(NodeValueImpl('impact', relatedThreatsWeights[1]));
        await impactNormal
            .addValue(NodeValueImpl('recuuid', recoNodeIterator.current.key));
        var shortNode = await recoNodeIterator.current.value.getValue('short');
        await impactNormal
            .addValue(NodeValueImpl('short', shortNode!.getValue('en')!));

        //RecommendationType == types.Devices ? coreValues.deviceRecommendation[relatedThreatsWeights[0]]!.add(impactNormal) : coreValues.userRecommendation[relatedThreatsWeights[0]]!.add(impactNormal);

        if (recommendationType == Types.devices.toLowerCase()) {
          try {
            CoreValues.deviceRecommendation[relatedThreatsWeights[0]]!
                .add(impactNormal);
          } catch (e) {
            var temp = <Node>[];
            temp.add(impactNormal);
            CoreValues.deviceRecommendation[relatedThreatsWeights[0]] = temp;
          }
        } else {
          try {
            CoreValues.userRecommendation[relatedThreatsWeights[0]]!
                .add(impactNormal);
          } catch (e) {
            var temp = <Node>[];
            temp.add(impactNormal);
            CoreValues.userRecommendation[relatedThreatsWeights[0]] = temp;
          }
        }
      }
    }
  } catch (e) {
    // ignore: avoid_print
    print(e);
    return;
  }
}

Future getMetrics(var type, var uuid) async {
  for (var pluginsUUID in Storage.pluginUUIDs) {
    //  print(':$type:' + UUID+':'+pluginsUUID+':data:metrics');
    try {
      var metricNode = await Storage.controller!
          .get(':$type:' + uuid + ':' + pluginsUUID + ':data:metrics');

      var childrenNodes = await metricNode.getChildren();

      var metricNodeIterator = childrenNodes.entries.iterator;

      while (metricNodeIterator.moveNext()) {
        var normalizationValue = await normalization(metricNodeIterator);

        var threatsImpactNode =
            await metricNodeIterator.current.value.getValue('threatsImpact');

        var threatsImpacts = threatsImpactNode!.getValue('en')!.split(';');

        for (var list in threatsImpacts) {
          var threatsImpact = list.split(',');

          //  print(metricNodeIterator.current.value.getValue('flag')!.getValue('en'));
          var flagNode =
              await metricNodeIterator.current.value.getValue('flag');

          if (flagNode!.getValue('en') == '1') {
            Node impactNormal = NodeImpl('', '');
            await impactNormal
                .addValue(NodeValueImpl('impact', threatsImpact[1]));
            await impactNormal.addValue(
                NodeValueImpl('normal', normalizationValue.toString()));

            //type == types.Devices ? coreValues.devicePositive[threatsImpact[0]]!.add(impactNormal) : coreValues.userPositive[threatsImpact[0]]!.add(impactNormal);

            if (type == Types.devices) {
              try {
                CoreValues.devicePositive[threatsImpact[0]]!.add(impactNormal);
              } catch (e) {
                var temp = <Node>[];
                temp.add(impactNormal);
                CoreValues.devicePositive[threatsImpact[0]] = temp;
              }
            } else {
              try {
                CoreValues.userPositive[threatsImpact[0]]!.add(impactNormal);
              } catch (e) {
                var temp = <Node>[];
                temp.add(impactNormal);
                CoreValues.userPositive[threatsImpact[0]] = temp;
              }
            }
          } else {
            Node impactNormal = NodeImpl('', '');
            await impactNormal
                .addValue(NodeValueImpl('impact', threatsImpact[1]));
            await impactNormal.addValue(
                NodeValueImpl('normal', normalizationValue.toString()));
            //   type == types.Devices ? coreValues.deviceNegative[threatsImpact[0]]!.add(impactNormal) : coreValues.userNegative[threatsImpact[0]]!.add(impactNormal);

            if (type == Types.devices) {
              try {
                CoreValues.deviceNegative[threatsImpact[0]]!.add(impactNormal);
              } catch (e) {
                var temp = <Node>[];
                temp.add(impactNormal);
                CoreValues.deviceNegative[threatsImpact[0]] = temp;
              }
            } else {
              try {
                CoreValues.userNegative[threatsImpact[0]]!.add(impactNormal);
              } catch (e) {
                var temp = <Node>[];
                temp.add(impactNormal);
                CoreValues.userNegative[threatsImpact[0]] = temp;
              }
            }
          }
        }
      }
    } catch (e) {
      continue;
    }
  }
}

Future getExistScoreNodes(var type, var path) async {
  var uuid = '';
  type == Types.users ? uuid = Storage.userUUID : uuid = Storage.deviceUUID;

  try {
    var strPath =
        ':$type:' + uuid + ':' + Storage.geigerIndicatorUUID + ':data:$path';
    var node = await Storage.controller!.get(strPath);

    if (path == Types.geigerScoreUser || path == Types.geigerScoreDevice) {
      CoreValues.existReco =
          node.getValue('implementedRecommendations').toString().split(',');
    }
  } catch (e) {
    //Creation
    // ignore: avoid_print
    print('\n');
    // ignore: avoid_print
    print('Node : $type is not found');

    if (path == Types.geigerScoreUser ||
        path == Types.geigerScoreDevice ||
        path == Types.geigerScoreAggregate) {
      // ignore: avoid_print
      print(path);
      await createScoreNode(type, path); //initailzation
    } else if (path == Types.geigerScoreMSE) {
      await createScoreMSENode(type, path);
    } else if (path == Types.recommendations) {
      await createRecommendationNode(type);
    }
  }
}

Future createRecommendationNode(var type) async {
  var uuid = '';

  type == Types.users ? uuid = Storage.userUUID : uuid = Storage.deviceUUID;

  Node recommendationNode = NodeImpl('recommendations', '',
      ':$type:' + uuid + ':' + Storage.geigerIndicatorUUID + ':data');

  await Storage.controller!.add(recommendationNode);
}

Future updateRecommendationNode(var type) async {
  var uuid = '';

  type == Types.users ? uuid = Storage.userUUID : uuid = Storage.deviceUUID;

  try {
    var recommendationNode = await Storage.controller!.get(':$type:' +
        uuid +
        ':' +
        Storage.geigerIndicatorUUID +
        ':data:recommendations');

    var threatRecoList = await crtRecommendation(type);

    for (NodeValue tr in threatRecoList) {
      await recommendationNode.addValue(tr);
    }

    //print(RecommendationNode);
    try {
      Storage.controller!.update(recommendationNode);
    } catch (e) {
      // ignore: avoid_print
      print(e);
    }
  } catch (e) {
    // ignore: avoid_print
    print(e);
    return;
  }
}

Future createScoreMSENode(var type, var path) async {
  await Storage.controller!
      .add(NodeImpl(Storage.geigerIndicatorUUID, ':$type'));
  await Storage.controller!
      .add(NodeImpl('data', ':$type:' + Storage.geigerIndicatorUUID));

  Node scoreMSENode =
      NodeImpl(path, ':$type:' + Storage.geigerIndicatorUUID + ':data');

  //create node keys: UserScore, ImplementRecommendations
  NodeValue geigerScore =
      NodeValueImpl('GEIGER_score', '0'); //add node name (name of threat)
  await scoreMSENode.addValue(geigerScore);

  NodeValue location = NodeValueImpl('location', '');
  await scoreMSENode.addValue(location);

  NodeValue sector = NodeValueImpl('sector', '0');
  await scoreMSENode.addValue(sector);

  NodeValue associatedProfiles = NodeValueImpl('associatedProfiles', '0');
  await scoreMSENode.addValue(associatedProfiles);

  geigerScore.setDescription('GEIGER user score');

  await Storage.controller!.add(scoreMSENode);
}

void updateScoreMSENode() async {
  try {
    if (CoreValues.flagMainUser) {
      var scoreMSENode = await Storage.controller!.get(':Enterprise:' +
          Storage.geigerIndicatorUUID +
          ':data:GeigerScoreMSE');

      //create node keys: UserScore, ImplementRecommendations
      // ignore: non_constant_identifier_names
      NodeValue GEIGERScore = NodeValueImpl('GEIGER_score',
          CoreValues.gagg.toString()); //add node name (name of threat)
      scoreMSENode.updateValue(GEIGERScore);

      NodeValue location =
          NodeValueImpl('location', CoreValues.enterpriseNode['location']);
      scoreMSENode.updateValue(location);

      NodeValue sector =
          NodeValueImpl('sector', CoreValues.enterpriseNode['sector']);
      scoreMSENode.updateValue(sector);

      NodeValue associatedProfiles = NodeValueImpl('associatedProfiles',
          CoreValues.enterpriseNode['associatedProfiles']);
      scoreMSENode.updateValue(associatedProfiles);

      await Storage.controller!.update(scoreMSENode);
      // print(ScoreMSENode);
    }
  } catch (e) {
    // ignore: avoid_print
    print(e);
  }
}

Future createScoreNode(var type, var path) async {
  var uuid = '';
  type == Types.users ? uuid = Storage.userUUID : uuid = Storage.deviceUUID;

  Node geigerIndicatorUserScoreNode = NodeImpl(
      path, '', ':$type:' + uuid + ':' + Storage.geigerIndicatorUUID + ':data');

  //create node keys: UserScore, ImplementRecommendations
  NodeValue geigerScore =
      NodeValueImpl('GEIGER_score', '0'); //add node name (name of threat)
  await geigerIndicatorUserScoreNode.addValue(geigerScore);

  NodeValue geigerThreatscores = NodeValueImpl('threats_score', '');
  await geigerIndicatorUserScoreNode.addValue(geigerThreatscores);

  NodeValue gegiernumMetrics = NodeValueImpl('number_metrics', '0');
  await geigerIndicatorUserScoreNode.addValue(gegiernumMetrics);

  NodeValue geigerJsonObject = NodeValueImpl('geigerJsonObj', '');
  await geigerIndicatorUserScoreNode.addValue(geigerJsonObject);

  geigerScore.setDescription('GEIGER user score');
  geigerThreatscores.setDescription('GEIGER threat-specific user score');
  gegiernumMetrics
      .setDescription('Number of metrics used in calculation of user score');

  if (path != Types.geigerScoreAggregate) {
    NodeValue implementedRecommendations = NodeValueImpl(
        'implementedRecommendations', ''); //add node name (name of threat)
    await geigerIndicatorUserScoreNode.addValue(implementedRecommendations);

    try {
      await Storage.controller!
          .add(NodeImpl(Storage.geigerIndicatorUUID, '', ':$type:' + uuid));
      await Storage.controller!.add(NodeImpl(
          'data', '', ':$type:' + uuid + ':' + Storage.geigerIndicatorUUID));
      await Storage.controller!.add(geigerIndicatorUserScoreNode);
    } catch (e) {
      //print(await storage.controller!.get(':$type:' + uuid + ':' + storage.GEIGERindicatorUUID + ':data:$path'));
    }
  } else {
    try {
      await Storage.controller!.add(geigerIndicatorUserScoreNode);
    } catch (e) {
      //print(e);
    }
  }
}

Future updateScoreNode(var type, var path) async {
  var uuid = '';
  type == Types.users ? uuid = Storage.userUUID : uuid = Storage.deviceUUID;

  try {
    var scoreNode = await Storage.controller!.get(
        ':$type:' + uuid + ':' + Storage.geigerIndicatorUUID + ':data:$path');

    await crtThreatScore(type);

    NodeValue geigerThreatscores = NodeValueImpl(
        'threats_score', jsonEncode(JsonObject.threatScoreValues));
    scoreNode.updateValue(geigerThreatscores);

    var geigerScore = 0.0;
    type == Types.geigerScoreAggregate
        ? geigerScore = CoreValues.gagg
        : geigerScore = await crtTotalThreatScore(type);

    NodeValue geigerScoreNode =
        NodeValueImpl('GEIGER_score', geigerScore.toString());
    scoreNode.updateValue(geigerScoreNode);

    JsonObject.geigerNode['threatScores'] = JsonObject.threatScoreValues;
    JsonObject.geigerNode['geigerScore'] = geigerScoreNode;

    var len = 0;
    type == Types.devices
        ? len =
            CoreValues.devicePositive.length + CoreValues.deviceNegative.length
        : len = CoreValues.userPositive.length + CoreValues.userNegative.length;

    var numberMetrics = '';
    type == Types.geigerScoreAggregate
        ? numberMetrics = CoreValues.nagg.toString()
        : numberMetrics = len.toString();

    NodeValue geigerNumMetrics = NodeValueImpl('number_metrics', numberMetrics);
    scoreNode.updateValue(geigerNumMetrics);

    if (path != Types.geigerScoreAggregate) {
      NodeValue implementedRecommendations =
          NodeValueImpl('implementedRecommendations', ''); //?????
      scoreNode.updateValue(implementedRecommendations);

      var objectName = '';
      type == Types.devices
          ? objectName = 'geigerScoreDevice'
          : objectName = 'geigerScoreUsers';
      JsonObject.geigerNodeKey[objectName] = JsonObject.geigerNode;

      type == Types.devices
          ? JsonObject.geigerScoreDevice = JsonObject.geigerNode
          : JsonObject.geigerScoreUser = JsonObject.geigerNode;
    } else {
      JsonObject.geigerNodeKey['geigerScoreAggregate'] = JsonObject.geigerNode;
      JsonObject.geigerScoreAggregate = JsonObject.geigerNode;
    }

    NodeValue geigerJsonObject =
        NodeValueImpl('geigerJsonObj', jsonEncode(JsonObject.geigerNodeKey));
    scoreNode.updateValue(geigerJsonObject);

    await Storage.controller!.update(scoreNode);

    //print(scoreNode);

  } catch (e) {
    // ignore: avoid_print
    print(e);
    return;
  }
}
